package kr.bit.model;

import java.sql.*;
import java.util.ArrayList;

// JDBC -> myBatis, JPA
public class MemberDAO {
	private Connection conn;
	private PreparedStatement ps;
	private ResultSet rs;
	
	// 데이터베이스 연결 객체 생성
	public void getConnect() {
		// 데이터베이스 접속 URL -> 데이터베이스 벤더마다 다름.
		String URL = "jdbc:mysql://localhost:3306/test?characterEncoding=UTF-8&serverTimeZone=UTC";
		String user = "root";
		String password = "1234";
		
		// MySQL Driver Loading
		try {
			// 동적 로딩. (실행 시점에서 객체를 생성하는 방법)
			Class.forName("com.mysql.jdbc.Driver");
			conn = DriverManager.getConnection(URL, user, password);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	// 회원 저장 동작
	public int memberInsert(MemberVO vo) {
		String SQL = "insert into member(id, pass, name, age, email, phone) values(?, ?, ?, ?, ?, ?)";	// ?는 파라미터를 의미.
		getConnect();
		
		// SQL 문장을 전송하는 객체 생성
		
		int cnt = -1;
		
		try {
			ps = conn.prepareStatement(SQL);	// 미리 컴파일 시킴(속도 향상)
			ps.setString(1, vo.getId());
			ps.setString(2, vo.getPass());
			ps.setString(3, vo.getName());
			ps.setInt(4, vo.getAge());
			ps.setString(5, vo.getEmail());
			ps.setString(6, vo.getPhone());
			
			// 성공 시 1, 실패 시 0
			cnt = ps.executeUpdate();					// 전송(실행)
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			dbClose();
		}
		
		return cnt;
	}
	
	// 회원(VO) 전체 리스트(ArrayList) 가져오기
	public ArrayList<MemberVO> memberList() {
		String SQL = "select * from member";
		getConnect();
		ArrayList<MemberVO> list = new ArrayList<MemberVO>();
		
		try {
			ps = conn.prepareStatement(SQL);		
			rs = ps.executeQuery();	// rs -> 커서
			
			while (rs.next()) {
				int num = rs.getInt("num");
				String id = rs.getString("id");
				String pass = rs.getString("pass");
				String name = rs.getString("name");
				int age = rs.getInt("age");
				String email = rs.getString("email");
				String phone = rs.getString("phone");
				
				// 묶고
				MemberVO vo = new MemberVO(num, id, pass, name, age, email, phone);
				
				// 담고
				list.add(vo);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			dbClose();
		}
		
		return list;
	}
	
	// 회원 삭제기능
	public int memberDelete(int num) {
		String SQL = "delete from member where num = ?";
		getConnect();
		int cnt = -1;
		try {
			ps = conn.prepareStatement(SQL);
			ps.setInt(1, num);
			cnt = ps.executeUpdate();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			dbClose();
		}
		
		return cnt;
	}
	
	// 회원 상세보기 기능
	public MemberVO memberContent(int num) {
		String SQL = "select * from member where num = ?";
		getConnect();
		MemberVO vo = null;
		try {
			ps = conn.prepareStatement(SQL);
			ps.setInt(1, num);
			rs = ps.executeQuery();
			
			if (rs.next()) {
				// 데이터 존재 시 회원 한명의 데이터 묶기(VO)
				num = rs.getInt("num");
				String id = rs.getString("id");
				String pass = rs.getString("pass");
				String name = rs.getString("name");
				int age = rs.getInt("age");
				String email = rs.getString("email");
				String phone = rs.getString("phone");
				
				vo = new MemberVO(num, id, pass, name, age, email, phone);
			}
		} catch(Exception e) {
			e.printStackTrace();
		} finally {
			dbClose();
		}
		
		return vo; 
	}
	
	// 회원정보 수정기능
	public int memberUpdate(MemberVO vo) {
		String SQL = "update member set age = ?, email = ?, phone = ? where num = ?";
		getConnect();
		int cnt = -1;
		try {
			ps = conn.prepareStatement(SQL);
			ps.setInt(1, vo.getAge());
			ps.setString(2, vo.getEmail());
			ps.setString(3, vo.getPhone());
			ps.setInt(4, vo.getNum());
			
			cnt = ps.executeUpdate();
		} catch(Exception e) {
			e.printStackTrace();
		}finally {
			dbClose();
		}
		
		return cnt;
	}
	
	// 데이터 베이스 연결 끊기
	public void dbClose() {
		try {
			if (rs != null) {
				rs.close();
			};
			
			if (ps != null) {
				ps.close();
			};
			
			if (conn != null) {
				conn.close();
			};			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
